/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.catalina.ssi;

import java.io.Serial;

/**
 * Exception used to tell SSIProcessor that it should stop processing SSI commands. This is used to mimic the Apache
 * behavior in #set with invalid attributes.
 */
public class SSIStopProcessingException extends Exception {

    /**
     * Default constructor.
     */
    public SSIStopProcessingException() {
        super();
    }

    /**
     * Constructor with cause.
     *
     * @param cause the underlying cause
     */
    public SSIStopProcessingException(Throwable cause) {
        super(cause);
    }

    @Serial
    private static final long serialVersionUID = 1L;
    // No specific functionality for this class
}