/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.tomcat.dbcp.pool2;

/**
 * Destroy context provided to object factories via {@code destroyObject} and {@code invalidateObject} methods. Values
 * provide information about why the pool is asking for a pooled object to be destroyed.
 *
 * @since 2.9.0
 */
public enum DestroyMode {

    /** Normal destroy. */
    NORMAL,

    /** Destroy abandoned object. */
    ABANDONED
}
